package taskman_lib;

# Copyright (C) 2000 Julian T. J. Midgley <jtjm@xenoclast.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# A copy of version 2 of the GNU General Public License may be found
# in the file "LICENCE" in the original Agnostos installation driectory.

# Provides task manager specific libraries

use strict;
use vars qw(@ISA @EXPORT);
use DBI;
use Sys::Hostname;

use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
	     html_header
	     html_footer
	     get_table
	     gen_assigned_name
	     optionlist
	     named_optionlist
	     sql_quote
	     fetch_ents
	     dbserver_connect
	     Do_SQL_Query
	     get_cookie
	     set_cookie
	     error_page
	     get_ent_names
	     get_config
	     sel_table_html
	     desalinate
	     check_passwd
	     gen_salt
	     parse_date_format
	     format_text_date
	    );

#--------------------------------------------------------------------------
# get_config([$config_file])
#
# Reads the config file ($CONFIG_FILE), and returns the configuration
# as a hash.

sub
get_config
{
    my $config_file = shift @_; # optional - overrides $CONFIG_FILE if present
    my $CONFIG_FILE = "config/agnostos.conf";
    $CONFIG_FILE = $config_file if (defined $config_file);
    my %config;

    open (IN,$CONFIG_FILE) 
	or die "AGNOSTOS: FATAL: Cannot open configuration file: $CONFIG_FILE";

    while (<IN>) {
	# Throw away comments and blank lines
	next if (/^\#.*|^\s+$|^\s+\#|^$/);
	
	# Check for valid key-value pair and extract key into $1, value into $2
	unless (/^\s*([a-zA-Z][a-zA-Z_0-9\-]*?)\s*=\s*(\S.*?)\s*$/) {
	    warn "AGNOSTOS: Warning - invalid line in config file:'$_'";
	    next;
	}
	
	if (defined $config{$1}) {
	    warn "AGNOSTOS: Warning - parameter '$1' defined more than once,".
		 "ignoring '$1'='$2'";
	    next;
	}
	$config{$1}=$2;
    }
    return %config;
}
	



#--------------------------------------------------------------------------
# get_cookie ($query)

sub
get_cookie 
{
    my $query = shift @_;
    my %tablehash = $query -> cookie ("AGNOSTOS-TABLEREF");
    unless (%tablehash) {
	$tablehash{'fubar'}='completely';
    }
    return \%tablehash;
}


#--------------------------------------------------------------------------
# set_cookie ($tableref, $query, $config_ref)

sub 
set_cookie 
{
    my ($tableref, $query, $config_ref) = @_;
    
    my $cookie_out;
    $cookie_out = $query -> cookie( -name    => 'AGNOSTOS-TABLEREF',
				    -expires => "+".$$config_ref{cookie_expiry_time}."d",
				    -path    => '/',
				    -value   => $tableref );
    return $cookie_out;
}

#--------------------------------------------------------------------------
# numeric sort
sub
numeric
{
    $a <=> $b;
}


#--------------------------------------------------------------------------
# Do_SQL_Query ($dbh, $query)


sub 
Do_SQL_Query 
{
    my $dbh = shift @_;
    my $query = shift @_ || return(0);
    my $dbq_query = $dbh -> prepare($query);
    my $num_rows = $dbq_query -> execute;
	
    return ( $dbq_query );
}


#--------------------------------------------------------------------------
# dbserver_connect( $database );
# Returns a handle to the database $database.

sub
dbserver_connect 
{
    my $config_ref = shift @_;
    my $type     = $$config_ref{dbase_type};
    my $user     = $$config_ref{dbase_user};
    my $host     = $$config_ref{dbase_host};
    my $database = $$config_ref{dbase_name};
    my $dbh = DBI->connect("DBI:$type:$database:$host", "$user")
	or die "Can't connect to $database database";
    return($dbh);
}

#--------------------------------------------------------------------------
# html_header($title,$query,@cookie)
sub
html_header 
{
    my ($title, $query, $cookie) = @_;
    my $ret;
    if ( defined $cookie ) { 
	$ret = $query -> header(-cookie=>$cookie,
			        -Pragma=>'no-cache');
    }
    else {
	$ret = $query -> header(-Pragma=>'no-cache');
    }
    $ret .= "\n<html><head><meta name=\"Pragma\" content=\"nocache\"><title>".
	    "$title</title></head><body bgcolor=white vlink=black>\n";
    return $ret;
}

#--------------------------------------------------------------------------
# html_footer()
sub
html_footer
{
    return "</body></html>\n";
}

#--------------------------------------------------------------------------
# gen_assigned_name($table)
# returns the appropriate name of the id column in the assigned table.

sub
gen_assigned_name
{

    my $table = shift @_;
    my $name;
    if ($table eq 'user') {
	$name = 'userid';
    }
    elsif ($table eq 'workgroup') {
	$name = 'groupid';
    }
    elsif ($table eq 'department') {
	$name = 'depid';
    }
    else {
	die "Invalid table name at gen_assigned_name($table)";
    }
}


#--------------------------------------------------------------------------
# get_table($table)
# Sets the database elements to use

sub
get_table
{
    my ($id, $name);
    my $table = shift @_;
    if ($table eq 'user') {
	$id   = 'userid';
	$name = 'username';
    }
    elsif ($table eq 'workgroup') {
	$id   = 'groupid';
	$name = 'groupname';
    }
    elsif ($table eq 'department') {
	$id   = 'depid';
	$name = 'depname';
    }
    else {
	die "Invalid table name: $table";
    }
    return ($id, $name);
}

#--------------------------------------------------------------------------
# optionlist ($default, @options)
# returns an HTML option list

sub 
optionlist 
{
    my $default = shift @_;
    $default = "" unless (defined $default);
    my @options = @_;
    my ($option, $str);
    $str = "";
    foreach $option (@options) {
	$str .= "<option ";
	if ( $option eq $default ) {
	    $str .= "selected ";
	}
	$str .= "value=$option>$option\n";
    }
    return $str;
}

#--------------------------------------------------------------------------
# named_optionlist($default, %options)
# returns an HTML option list, where the values and options differ

sub 
named_optionlist 
{
    my $default = shift @_;
    $default = "" unless (defined $default);
    my %options = @_;
    my ($option, $str);
    $str = "";
    foreach $option (sort numeric keys %options) {
	$str .= "<option ";
	if ( $option eq $default ) {
	    $str .= "selected ";
	}
	$str .= "value=$option>$options{$option}\n";
    }
    return $str;
}

#--------------------------------------------------------------------------
# sql_quote($string) 
#
# Returns $string with all special characters (like
# single quotation marks), commented out

sub
sql_quote
{
    my $string = shift @_;
    $string =~ s/\'/\'\'/g;
    return $string;
}

#--------------------------------------------------------------------------
# fetch_ents($table, $dbh) 
# returns a hash of the entity ids and names, keyed by id.

sub
fetch_ents 
{
    my ($table, $dbh) = @_;
    my ($entid, $entname, %entities);
    my ($id, $name) = get_table($table);
    my $dbq = $dbh -> prepare("SELECT $id, $name FROM $table");
    $dbq -> execute;
    while ( ($entid, $entname) = $dbq -> fetchrow_array ) {
	if ($entname eq 'UNSET') {
	    $entname = "&nbsp;&nbsp;&nbsp; -- &nbsp;&nbsp;&nbsp;";
	}
	$entities{$entid} = $entname;
    }
    $dbq -> finish;
    return %entities;
}

#--------------------------------------------------------------------------
# error_page( $err_string, $query )
# Display an error page

sub
error_page
{
    my ($err_string, $query) =  @_;
    my $page;
    $page  = html_header("Error", $query);
    $page .= "<h2>Error</h2>$err_string\n";
    $page .= html_footer();
    return $page;
}

#--------------------------------------------------------------------------
# get_ent_names ($tableref, $dbh) 
# Returns a string containing the names of the selected tables passed
# in $tableref

sub
get_ent_names
{
    my ($tableref, $dbh) = @_;
    my $ent_table;
    my $ent_name = "";
    foreach $ent_table (keys %$tableref) {
	my ($act_id, $act_name) = get_table($ent_table);
	my $dbq = $dbh -> prepare("SELECT $act_name 
                                   FROM $ent_table 
                                   WHERE $act_id = $$tableref{$ent_table}");
	$dbq -> execute;
	if (length($ent_name) > 0) {
	    $ent_name .= " &amp; ";
	}
	$ent_name .= $dbq -> fetchrow_array;
	$dbq -> finish;
    }    
    return $ent_name;
}


#--------------------------------------------------------------------------
# sel_table_html($caption, $tableref, $dbh, $config_ref)
# Returns an HTML table of entity names 

sub
sel_table_html
{
    my ($caption, $tableref, $dbh, $config_ref) = @_;
    my ($ent_table, %selected_tables, $html);
    foreach $ent_table (keys %$tableref) {
	my ($act_id, $act_name) = get_table($ent_table);
	my $dbq = $dbh -> prepare("SELECT $act_name 
                                   FROM $ent_table 
                                   WHERE $act_id = $$tableref{$ent_table}");
	$dbq -> execute;
	$selected_tables{$ent_table} = $dbq -> fetchrow_array;
    }
    my $td = "<td bgcolor=$$config_ref{col_selected_table_bg} align=center>";

    $html = "<table border=3 rules=none cellspacing=0 cellpadding=3>".
	    "<caption><b>$caption</b></caption><tr>"; 
    foreach $ent_table ('user','workgroup','department'){
	if (defined $selected_tables{$ent_table}) {
	    $html .= $td."<font size=+1 color=$$config_ref{col_selected_table_fg}>".ucfirst($ent_table)."</font>";
	}
    }
    $html .= "\n<tr>";
    foreach $ent_table ('user','workgroup','department'){
	if (defined $selected_tables{$ent_table}) {
	    $html .= $td."<font size=+1 color=$$config_ref{col_selected_table_fg}><b>".$selected_tables{$ent_table}."</b></font>";
	}
    }
    $html .= "\n</table>\n";
    return $html;
}

#--------------------------------------------------------------------------
# desalinate ($pwdhash)
# Returns the salt of $pwdhash (for either DES or MD5 crypt()ed passwords)

sub
desalinate 
{
    my $pwdhash = shift @_;
    if ( $pwdhash =~ /^(\$1\$.*\$)/ ) {
	# MD5 
	return $1;
    }
    else {
	# DES
	return substr($pwdhash, 0, 2);
    }
}

#--------------------------------------------------------------------------
# check_passwd ($passwd, $pwdhash);
# $passwd is the password attempt, $pwdhash is the crypt()ed hash
# to validate $passwd against.
# Returns true if they match, false otherwise

sub
check_passwd
{
    my ($passwd, $pwdhash) = @_;
    my $salt = desalinate($pwdhash);
    return (crypt ($passwd, $salt) eq $pwdhash);
}

#--------------------------------------------------------------------------
# gen_salt($alg);
# Generates a random salt for use with crypt().
# $alg should be either 'MD5' or 'DES'
# NB - this algorithm is currently sub-optimal (it doesn't make use of the entire
# keyspace
sub 
gen_salt
{
    ;
}

#--------------------------------------------------------------------------
# parse_date_format($config_ref)
#
# returns ($sep, @date_format) where $sep is the preferred separator,
# and @date_format describes the relative ordering of Day, Month and Year.
 
sub
parse_date_format
{
    my $config_ref = shift @_;
    my @valid_formats = ('DMY','DYM','YMD','YDM','MDY','MYD');
    my (@date_format, $sep, $poss);
    if ( $$config_ref{date_format} =~ /(\w)(\W)(\w)\W(\w)/ ) {
	$sep = $2;
	@date_format = ($1, $3, $4);
	
	# Check date format is valid
	my $format = join('',@date_format);
	if ( scalar( grep(/$format/, @valid_formats) ) < 1 ) {
	    die ("Configuration option date_format '$$config_ref{date_format}' is invalid");
	}
	return ($sep, @date_format)
    }
    else {
	die ("Configuration option date_format '$$config_ref{date_format}' is invalid");
    }
}

#--------------------------------------------------------------------------
# format_text_date ($day, $month, $year, $sep, @date_format);

sub 
format_text_date 
{
    my @date;
    for (my $i=0; $i<3; $i++) {
	push (@date, shift @_);
    }
    my ($sep, @date_format) = @_;
    my %cunning = ( D => 0,
		    M => 1,
		    Y => 2 );
    
    my (@formatted,$element);
    
    foreach $element (@date_format) {
	push (@formatted ,$date[$cunning{$element}]);
    }
    
    return join($sep,@formatted);
}
	
1;
    
