#!/usr/bin/perl -w

# Copyright (C) 2000 Julian T. J. Midgley <jtjm@xenoclast.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# A copy of version 2 of the GNU General Public License may be found
# in the file "LICENCE" in the original Agnostos installation driectory.

use strict;
use lib "agnostos_docroot";
use taskman_lib;
use Carp;

my $config_file = "agnostos_docroot/config/agnostos.conf";
my %config = get_config($config_file);

# Prompt for name of database
print "\nAgnostos Installation\n".
      "---------------------\n\n".
      "See the README file for instructions\n\n";

print "Installation directory [".$config{agnostos_home}."]:\n";
my $install_dir = check_option ( scalar(<>), $config{agnostos_home} );

my $upgrade = check_install_dir($install_dir);

unless ($upgrade) {
    print "Agnostos database name [".$config{dbase_name}."]:\n";
    my $dbase_name = check_option( scalar(<>), $config{dbase_name} );

    print "Agnostos user name [".$config{dbase_user}."]:\n";
    my $user_name = check_option ( scalar(<>), $config{dbase_user} );
    
    # Install new copy 
    db_install( $dbase_name, $user_name );
    # Copy files
    print "Copying files to $install_dir...";
    system ("cp -a agnostos_docroot $install_dir");
    print "done\n";
    # Update agnostos.conf
    print "Updating configuration file agnostos.conf...";
    update_config( $install_dir, $dbase_name, $user_name );
    print "done\n";
    print "\nInstallation Complete\n";
}
    
exit 0;


#--------------------------------------------------------------------------
# check_option ($choice, $default);
#
# If $choice is anything other than a null string, returns $choice,
# else returns $default

sub
check_option
{
    my ($choice, $default) = @_;
    chomp ($choice);
    if (length($choice) == 0) {
	return $default;
    }
    else 
    {
	return $choice;
    }
}

#--------------------------------------------------------------------------
# get_version($version_file) 

sub
get_version
{
    my $version_file = shift @_;
    open( IN, $version_file ) 
	or die "Cannot open version file $version_file";
    my $version = <IN>;
    close IN;
    chomp( $version );
    return $version;
}

#--------------------------------------------------------------------------
# yn_prompt($prompt, $default)

sub
yn_prompt
{
    my ($prompt, $default) = @_;
    chomp ($prompt);
    my $response;
    my $ok = 0;

    while ( ! $ok ) {
	print $prompt."\n";
	$response = <>;
	$ok = (( $response =~ /^[yYnN]$/ ) or ( $response =~ /^$/ )) ? 1 : 0;
    }
    $response = $default if ( $response =~ /^$/ );
    return ( $response =~ /^[yY]$/ );
} 

#--------------------------------------------------------------------------
# update_config( $install_dir, $dbase_name, $dbase_user );

sub
update_config
{
    my ($install_dir, $dbase_name, $dbase_user) = @_;
    my $config_file = "$install_dir/config/agnostos.conf";
    my $outfile = $config_file.$$;
    open( IN, $config_file ) 
	or confess "Cannot open config file: $config_file";
    open( OUT, ">$outfile" )
	or confess "Cannot open $outfile for writing";
    while(<IN>) {
	s/^(dbase_name = ).*$/$1$dbase_name/;
	s/^(dbase_user = ).*$/$1$dbase_user/;
	s/^(agnostos_home = ).*$/$1$install_dir/;
	print OUT;
    }
    close OUT;
    close IN;
    rename($outfile, $config_file) 
	or die "FATAL: could not rename $outfile to $config_file\n".
	    "Config file not updated.";
}
#--------------------------------------------------------------------------
# db_install( $dbase_name, $user_name )

sub
db_install
{
    my ($dbase_name, $user_name) = @_;
    open( IN, "install_mysql.sh" ) 
	or die "Cannot open install_mysql.sh";
    open( OUT, ">install_mysql_real.sh" )
	or die "Cannot open install_mysql_real.sh for writing";
    while ( <IN> ) {
	s/exit 1//;
	s/^DBASE_NAME=.*$/DBASE_NAME=$dbase_name\n/;
	print OUT;
    }
    close OUT;
    close IN;
    
    open( IN, "mysql_data/mysql_inserts" ) 
	or die "Cannot open mysql_date/mysql_inserts";
    open( OUT, ">mysql_data/mysql_inserts_real" )
	or die "Cannot open mysql_data/mysql_inserts_real for writing";
    while ( <IN> ) {
	s/^(INSERT INTO user.*)agnostos/$1$user_name/;
	s/^(INSERT INTO db.*)agnostos/$1$dbase_name/;
	print OUT;
    }
    close OUT;
    close IN;

    # Execute installer
    chmod( 0755, "install_mysql_real.sh" );
    system( "./install_mysql_real.sh" );
}

#--------------------------------------------------------------------------
# check_install_dir($install_dir)
#
# Checks for the existence of $install_dir - if it finds it, asks
# whether or not to upgrade. 
#
# If must upgrade - returns version number of previous version
# else returns 0 => install brand new copy

sub 
check_install_dir
{
    my $install_dir = shift @_;
    my ($pre_version, $upgrade);
    my $version = get_version("agnostos_docroot/VERSION");
    if ( -e $install_dir ) {
	if ( -d $install_dir ) {
	    # Directory already exists

	    if ( -f "$install_dir/taskman_lib.pm" ) {

		# Previous version of Agnostos installed 
		if ( -f "$install_dir/VERSION" ) {
		    $pre_version = get_version("$install_dir/VERSION");
		}
		else {
		    $pre_version = "pre_0.9.9";
		}
		$upgrade = yn_prompt("Upgrade from $pre_version to $version (y/n)? [y]", 'y');
		unless ( $upgrade ) {
		    print "Aborting Agnostos installation\n";
		    exit 1;
		}
	    }
	    else {
		# Something else there
		print "FATAL: $install_dir already exists, ".
		      "but doesn't contain a previous \n version of Agnostos.\n";
		exit 1;
	    }
	    
	    # Upgrade from an earlier version
	    print "Upgrading\n";
	    my $backup_dir = $install_dir.".old";
	    print "Moving $install_dir to $backup_dir...";
	    if (-e $backup_dir ) {
		die "Ugrade aborted: $backup_dir already exists - please\n".
		    "move it out of the way, and run install.pl again.";
	    }
    
	    rename( $install_dir, $backup_dir ) 
		or die "Could not rename $install_dir to $backup_dir";
	    print "done\n";

	    # Do upgrade
	    print "Copying files to $install_dir...";
	    system ("cp -a agnostos_docroot $install_dir");
	    print "done\n";

	    print "Merging old and new configurations (old config will be preserved)...";
	    my $merged_config_ref = merge_config( $backup_dir, $install_dir );
	    print "done\n";

	    # Write the merged configuration to the new config file
	    print "Writing merged configuration to $install_dir/config/agnostos.conf...";
	    write_config ($merged_config_ref, "$install_dir/config/agnostos.conf");
	    print "done\n";

	    print "Upgrade complete\n";
	    return $pre_version;
	}
	print "FATAL: $install_dir already exists, and is not a directory\n";
	print "Aborting Agnostos installation\n";
	exit 1;
    }
    # Install fresh copy
    return 0;
}

#--------------------------------------------------------------------------
# merge_config( $dir_old, $dir_new )

sub
merge_config
{
    my ($dir_old, $dir_new) = @_;
    my %old_config = get_config("$dir_old/config/agnostos.conf");
    my %new_config = get_config("$dir_new/config/agnostos.conf");
    my (%merged_config, $key, $value);
    
    foreach $key (keys %new_config) {
	if ( defined $old_config{$key} ) {
	    $merged_config{$key} = $old_config{$key};
	}
	else {
	    $merged_config{$key} = $new_config{$key};
	}
    }
    return \%merged_config;
}

#--------------------------------------------------------------------------
# write_config ($config_ref, $config_file);

sub
write_config
{
    my ($config_ref, $config_file) = @_;
    my %config = %$config_ref;
    my $temp_file = $config_file.".".$$;
    my $key;

    open( IN, $config_file ) 
	or die "Unable to open $config_file";
    open( OUT, ">$temp_file" ) 
	or die "Unable to open $temp_file for writing";
    while ( <IN> ) {
	if (/^\#.*|^\s+$|^\s+\#|^$/) {
	    # Comment
	    print OUT;
	    next;
	}	
	# Check for valid key-value pair and extract key into $1, value into $2
	unless (/^\s*([a-zA-Z][a-zA-Z_0-9\-]*?)\s*=\s*(\S.*?)\s*$/) {
	    warn "Disregarding invalid line in old config file :$_";
	    next;
	}
	
	$key = $1;
	print OUT "$key = ".$config{$key}."\n";
    }
    close OUT;
    close IN;
    rename( $temp_file, $config_file ) 
	or die "Could not rename $temp_file to $config_file";
}
	    
